###################################################################################################
# _versionmanager.py
#
# Implementation of class VersionManager (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTMLFile
import string 
import time
import copy
import urllib
# Product Imports.
import _globals 
import _blobfields 


# DEPRECATED: Products.zms._versionmanager.MyFile
class MyFile(_blobfields.MyFile): pass
# DEPRECATED: Products.zms._versionmanager.MyImage
class MyImage(_blobfields.MyImage): pass


# ---------------------------------------------------------------------------------------------
#  _versionmanager.resetObjVersion:
#
#  Resets work-version of object-attributes.
# ---------------------------------------------------------------------------------------------
def resetObjVersion(self):
  resetObjState(self)
  self.__work_version__ = _globals.MyClass()
  self.cloneObjAttrs(self,self.__work_version__)

# ---------------------------------------------------------------------------------------------
#  _versionmanager.resetObjState:
#
#  Resets work-state of given object.
# ---------------------------------------------------------------------------------------------
def resetObjState(self, REQUEST=None):
  if REQUEST is None or self.getAutocommit():
    self.__work_state__ = _globals.MyClass()
  else:
    states = self.getWorkStates()
    for obj_state in ['STATE_NEW','STATE_MODIFIED','STATE_MODIFIED_SILENT','STATE_DELETED','STATE_REQUESTED','STATE_ROLLEDBACK','STATE_REJECTED']:
      if obj_state in states:
        del states[states.index(obj_state)]
      elif REQUEST is not None and REQUEST.has_key('lang'):
        if getObjStateName(obj_state,REQUEST['lang']) in states:
          del states[states.index(getObjStateName(obj_state,REQUEST['lang']))]
    self.__work_state__.states = copy.deepcopy(states)
    self.__work_state__ = copy.deepcopy(self.__work_state__)

# ---------------------------------------------------------------------------------------------
#  _versionmanager.setObjWorkflowEventChange:
#
#  Applies information about user-ID and date of workflow-event.
# ---------------------------------------------------------------------------------------------
def setObjWorkflowEventChange(self, REQUEST):
  lang = REQUEST['lang']
  self.setObjProperty('work_uid',REQUEST['AUTHENTICATED_USER'].getId(),lang)
  self.setObjProperty('work_dt',time.time(),lang)

# ---------------------------------------------------------------------------------------------
#  _versionmanager.setObjStateChange:
#
#  Applies information about user-ID and date of change.
# ---------------------------------------------------------------------------------------------
def setObjStateChange(self, REQUEST):
  #++ print "[%s.setObjStateChange]: "%(self.meta_type)
  lang = REQUEST['lang']
  self.setObjProperty('change_uid',REQUEST['AUTHENTICATED_USER'].getId(),lang)
  self.setObjProperty('change_dt',time.time(),lang)

# ---------------------------------------------------------------------------------------------
#  _versionmanager.setObjState:
#
#  Appends work-state to given object.
# ---------------------------------------------------------------------------------------------
def setObjState(self, state, lang=None):

  # Apply information about modification (language-state, change-userid and change-date).
  if lang is not None:
    state = getObjStateName(state,lang)
  #++ print "[%s.setObjState]: state=%s"%(self.meta_type,state)
    
  # Append state.
  states = self.getWorkStates()
  if not state in states:
    states.append(state)
  self.__work_state__.states = copy.deepcopy(states)
  self.__work_state__ = copy.deepcopy(self.__work_state__)

# ---------------------------------------------------------------------------------------------
#  _versionmanager.getObjStateName:
#
#  Returns name object state in given language.
# ---------------------------------------------------------------------------------------------
def getObjStateName(state, lang):
  return '%s_%s'%(state,string.upper(lang))

# ---------------------------------------------------------------------------------------------
#  _versionmanager.doAutocommit:
#
#  Commit pending changes of all objects.
# ---------------------------------------------------------------------------------------------
def doAutocommit(self): 
  print "[%s.doAutocommit]: %s"%(self.meta_type,self.absolute_url())
  ##### Auto-Commit ####
  for s_lang in self.getLanguages():
    req = {}
    req['lang'] = s_lang
    self.__autoCommit__(req)
  ##### Process child-objects ####
  for ob in self.getChildNodes():
    doAutocommit(ob)



###################################################################################################
###################################################################################################
###
###   C l a s s   V e r s i o n M a n a g e r :
###
###################################################################################################
###################################################################################################
class VersionManager: 

    # Management Interface.
    version_input_fields = HTMLFile('dtml/versionmanager/version_input_fields', globals())
    version_object_state = HTMLFile('dtml/versionmanager/version_object_state', globals())

    # ---------------------------------------------------------------------------------------------
    #  VersionManager.getObjVersion:
    #
    #  Live or work-version of object-attributes.
    # ---------------------------------------------------------------------------------------------
    def getObjVersion(self, REQUEST=None):
      preview = _globals.isPreviewRequest(REQUEST)
      if preview:
        if not hasattr(self,'__work_version__'):
          resetObjVersion(self)
      	return self.__work_version__
      else:
        return self

    # ---------------------------------------------------------------------------------------------
    #  VersionManager.getObjectDisplay:
    #
    #  Checks if object can by displayed.
    # ---------------------------------------------------------------------------------------------
    def getObjectDisplay(self, REQUEST):

      #-- [ReqBuff]: Fetch buffered value from Http-Request.
      try:
        reqBuffId = 'getObjectDisplay'
        return self.fetchReqBuff(reqBuffId,REQUEST)
      except:

        #-- Get value.
        preview = _globals.isPreviewRequest(REQUEST)
        value = (self.getAutocommit()) or \
                (not preview and not self.inObjStates(['STATE_NEW'],REQUEST)) or \
                (preview and not self.inObjStates(['STATE_DELETED'],REQUEST))
                
        #-- [ReqBuff]: Returns value and stores it in buffer of Http-Request.
        return self.storeReqBuff(reqBuffId,value,REQUEST)


    """
    ###############################################################################################
    #
    #  Object State
    #
    ###############################################################################################
    """
    
    # ---------------------------------------------------------------------------------------------
    # 	IN (Object States)
    # ---------------------------------------------------------------------------------------------
    def inObjStates(self, obj_states, REQUEST):
      states = self.getObjStates()
      if len(states) > 0:
        lang = self.getPrimaryLanguage()
        if REQUEST is not None and REQUEST.has_key('lang'):
          lang = REQUEST['lang']
        for obj_state in obj_states:
          if obj_state in states or getObjStateName(obj_state,lang) in states:
            return 1
      return 0

    # ---------------------------------------------------------------------------------------------
    # 	getWorkStates
    # ---------------------------------------------------------------------------------------------
    def getWorkStates(self):
      if not hasattr(self,'__work_state__'):
        self.__work_state__ = _globals.MyClass()
      return getattr(self.__work_state__,'states',[])

    # ---------------------------------------------------------------------------------------------
    # 	GET (Object States)
    # ---------------------------------------------------------------------------------------------
    def getObjStates(self, REQUEST=None):
      states = self.getWorkStates()
      if self.getAutocommit():
        return []
      if REQUEST is None:
        obj_states = states
      else:
        obj_states = []
        lang = None
        if REQUEST is not None and REQUEST.has_key('lang'):
          lang = REQUEST['lang']
        for obj_state in ['STATE_NEW','STATE_MODIFIED','STATE_MODIFIED_SILENT','STATE_DELETED']:
          if obj_state in states:
            obj_states.append(obj_state)
          elif lang is not None:
            if getObjStateName(obj_state,lang) in states:
              obj_states.append(getObjStateName(obj_state,lang))
      # Special Objects.
      if self.meta_type == 'ZMSCustom':
        for child in self.getObjChildren('',REQUEST):
          child_obj_states = child.getObjStates(REQUEST)
          for child_obj_state in child_obj_states:
            if not child_obj_state in obj_states:
              obj_states.append(child_obj_state)
      # Return Object States.
      return obj_states
      
    # ---------------------------------------------------------------------------------------------
    # 	GET (Modified Silent)
    # ---------------------------------------------------------------------------------------------
    def getObjStateModifiedSilent(self):
      states = self.getWorkStates()
      return (len(states) == 0) or \
      	     (len(states) == 1 and string.find(states[0],'STATE_MODIFIED_SILENT')==0)
      
    # ---------------------------------------------------------------------------------------------
    # 	SET (Modified Silent)
    # ---------------------------------------------------------------------------------------------
    def setObjStateModifiedSilent(self, REQUEST):
      setObjState(self,'STATE_MODIFIED_SILENT',REQUEST['lang'])
      self.resetObjWorkflowEvents(REQUEST)
      
    # ---------------------------------------------------------------------------------------------
    # 	SET (Modified)
    # ---------------------------------------------------------------------------------------------
    def setObjStateModified(self, REQUEST):
      setObjStateChange(self,REQUEST)
      setObjState(self,'STATE_MODIFIED',REQUEST['lang'])
      self.resetObjWorkflowEvents(REQUEST)

    # ---------------------------------------------------------------------------------------------
    # 	SET (Deleted)
    # ---------------------------------------------------------------------------------------------
    def setObjStateDeleted(self, REQUEST):
      setObjStateChange(self,REQUEST)
      setObjState(self,'STATE_DELETED',REQUEST['lang'])
      self.resetObjWorkflowEvents(REQUEST)
        
    # ---------------------------------------------------------------------------------------------
    # 	SET (New)
    # ---------------------------------------------------------------------------------------------
    def setObjStateNew(self, REQUEST):
      resetObjVersion(self)
      setObjStateChange(self,REQUEST)
      setObjState(self,'STATE_NEW',REQUEST['lang'])
      self.resetObjWorkflowEvents(REQUEST)

    """   
    ###############################################################################################
    #
    #   C O M M I T
    #
    ###############################################################################################
    """   

    # ---------------------------------------------------------------------------------------------
    #  VersionManager.__autoCommit__:
    # ---------------------------------------------------------------------------------------------
    def __autoCommit__(self, REQUEST):
      #++ print '[%s.__autoCommit__]'%(self.meta_type)
          
      ##### Catalog ####
      self.synchronizeSearch(REQUEST)

      ##### Cache ####
      self.synchronizeCachePage(REQUEST)

      ##### Auto-Commit ####          
      if self.getAutocommit():
        parent = self.getParentNode()        
        delete = self._commitObjectVersionChanges(parent,REQUEST)
        if delete: 
          parent.normalizeSortIds()

    # ---------------------------------------------------------------------------------------------
    #  VersionManager._commitObjectVersionChanges
    # ---------------------------------------------------------------------------------------------
    def _commitObjectVersionChanges(self, parent, REQUEST):
      #++ print "[%s._commitObjectVersionChanges]: id=%s, states=%s"%(self.meta_type,self.id,str(self.getObjStates(REQUEST)))
      delete = 0
      ##### Commit delete object. ####
      if self.inObjStates(['STATE_DELETED'],REQUEST):
        if self.id in parent.objectIds():
          parent.manage_delObjects([self.id],None)
          delete = 1
      ##### Commit modifications. ####
      else:
        self.cloneObjAttrs(self.__work_version__,self,REQUEST)
        # Special Objects.
        if self.meta_type == 'ZMSCustom':
          for child in self.getObjChildren('',REQUEST):
            child._commitObjectVersionChanges(self,REQUEST)
      # Reset object-state.
      resetObjState(self,REQUEST)
      # Return flag for deleted objects.
      return delete
            
      
    """   
    ###############################################################################################
    #
    #   R O L L B A C K
    #
    ###############################################################################################
    """   
    
    # ---------------------------------------------------------------------------------------------
    #  VersionManager._rollbackObjectVersionChanges
    # ---------------------------------------------------------------------------------------------
    def _rollbackObjectVersionChanges(self, parent, REQUEST):
      delete = 0
      user = REQUEST['AUTHENTICATED_USER']
      if user.getId() == self.getObjProperty('change_uid',REQUEST) or \
         user.has_role('Manager') or \
         user.has_role('ZMSAdministrator') or \
         user.has_role('ZMSEditor'):
        #++ print "[%s._rollbackObjectVersionChanges]: id=%s, states=%s"%(self.meta_type,self.id,str(self.getObjStates(REQUEST)))
        ##### Rollback insert object. ####
        if self.inObjStates(['STATE_NEW'],REQUEST):
          if self.id in parent.objectIds():
            parent.manage_delObjects([self.id],None)
            delete = 1
        ##### Rollback modifications. ####
        else:            
          self.cloneObjAttrs(self,self.__work_version__,REQUEST)
          # Special Objects.
          if self.meta_type == 'ZMSCustom':
            for child in self.getObjChildren('',REQUEST):
              child._rollbackObjectVersionChanges(self,REQUEST)
        # Reset object-state.
        resetObjState(self,REQUEST)
      # Return flag for deleted objects.
      return delete

        
    """   
    ###############################################################################################
    #
    #   R E Q U E S T
    #
    ###############################################################################################
    """   
        
    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer._requestVersionChanges
    # ---------------------------------------------------------------------------------------------
    def _requestVersionChanges(self, parent, lang, manage_lang, REQUEST):
      s = ''
      if len(self.getObjStates()) > 0:
        s = s + '\n* %s: '%self.display_type(REQUEST) 
        for state in ['STATE_NEW','STATE_MODIFIED','STATE_DELETED']:
          if self.inObjStates([state],REQUEST):
            s = s + '[%s] '%self.getLangStr(state,manage_lang)
        s = s + '%s %s '%(self.getLangStr('ON',manage_lang),self.getLangFmtDate(self.getObjProperty('change_dt',REQUEST),manage_lang,'DATE_FMT'))
        s = s + '%s %s '%(self.getLangStr('AT',manage_lang),self.getLangFmtDate(self.getObjProperty('change_dt',REQUEST),manage_lang,'TIME_FMT'))
        s = s + '%s %s '%(self.getLangStr('BY',manage_lang),self.getObjProperty('change_uid',REQUEST))
      return s
        
        
###################################################################################################
###################################################################################################
###
###   C l a s s   V e r s i o n M a n a g e r C o n t a i n e r:
###
###################################################################################################
###################################################################################################
class VersionManagerContainer: 

    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.resetObjWorkflowEvents:
    #
    #  Resets information about workflow-events.
    # ---------------------------------------------------------------------------------------------
    def resetObjWorkflowEvents(self, REQUEST):
      lang = REQUEST['lang']
      self.setObjProperty('work_uid',None,lang)
      self.setObjProperty('work_dt',None,lang)
      if self.getAutocommit():
        self.__work_state__ = _globals.MyClass()
      else:
        states = getattr(self.__work_state__,'states',[])
        for state in ['STATE_REQUESTED','STATE_REJECTED','STATE_ROLLEDBACK']:
          if getObjStateName(state,lang) in states:
            del states[states.index(getObjStateName(state,lang))]
        self.__work_state__.states = copy.deepcopy(states)
        self.__work_state__ = copy.deepcopy(self.__work_state__)
      
    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.normalizeSortIdsAfterDelete:
    #
    #  Resets child-objects after delete..
    # ---------------------------------------------------------------------------------------------
    def normalizeSortIdsAfterDelete(self):
      self.normalizeSortIds()
      # Resets information about silent-modifications.
      for child in self.getChildNodes():
        states = getattr(child.__work_state__,'states',[])
        for state in ['STATE_MODIFIED_SILENT']:
          if state in states:
            del states[states.index(state)]
        child.__work_state__.states = copy.deepcopy(states)
        child.__work_state__ = copy.deepcopy(child.__work_state__)

    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.getContainerObjState:
    # ---------------------------------------------------------------------------------------------
    def getContainerObjState(self, REQUEST):
      state = self.inObjStates(['STATE_NEW','STATE_MODIFIED','STATE_DELETED'],REQUEST)
      for child in self.getChildNodes(REQUEST):
        state = state + len(child.getObjStates(REQUEST=REQUEST)) > 0
      return state


    """
    ###############################################################################################    
    #
    #   T A S K S 
    #
    ###############################################################################################    
    """

    # Management Interface.
    manage_tasks = HTMLFile('dtml/versionmanager/manage_tasks', globals()) 
    tasklist0 = HTMLFile('dtml/versionmanager/tasklist0', globals()) 
    tasklist1 = HTMLFile('dtml/versionmanager/tasklist1', globals()) 
    tasklist2 = HTMLFile('dtml/versionmanager/tasklist2', globals()) 
    tasklist3 = HTMLFile('dtml/versionmanager/tasklist3', globals()) 
    tasklist4 = HTMLFile('dtml/versionmanager/tasklist4', globals()) 
    
    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.getTasks:
    # ---------------------------------------------------------------------------------------------
    def getTasks(self, task_type, REQUEST):
      rtn = []
      append = 1
      if task_type == 0:
        append = append and self.getContainerObjState(REQUEST)
      elif task_type == 1:
        append = append and self.inObjStates(['STATE_REQUESTED'],REQUEST)
      elif task_type == 2:
        append = append and self.inObjStates(['STATE_REJECTED'],REQUEST)
      elif task_type == 3:
        append = append and len(self.getChildNodes(REQUEST,['ZMSNote'])) > 0
      elif task_type == 4:
        translate = len(self.getObjProperty('change_uid',REQUEST)) == 0
        for child in self.getChildNodes(REQUEST):
          translate = translate or len(child.getObjProperty('change_uid',REQUEST)) == 0
        append = append and self.getDCCoverage(REQUEST) in ['obligation','proposal'] and translate
      if append: rtn.append(self)
      for child in self.getChildNodes(REQUEST,self.PAGES):
        rtn.extend(child.getTasks(task_type,REQUEST))
      return rtn
    

    """   
    ###############################################################################################
    #
    #   C O M M I T
    #
    ###############################################################################################
    """   

    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.__autoCommitOnlyModifiedSilent__:
    #
    #  Auto commit container if it is not modified and nothing but silently
    #  modified objects (sort_id).
    # ---------------------------------------------------------------------------------------------
    def __autoCommitOnlyModifiedSilent__(self):
      #++ print '[%s.__autoCommitOnlyModifiedSilent__]'%(self.meta_type)
      req = {}
      req['lang'] = self.getPrimaryLanguage()
      req['preview'] = 'preview'
      flag = self.getObjStateModifiedSilent()      
      for child in self.getChildNodes(req):
        flag = flag and child.getObjStateModifiedSilent()
      flag = flag or self.getAutocommit()
      if flag:
        self._commitVersionChanges()

    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer._commitVersionChanges
    # ---------------------------------------------------------------------------------------------
    def _commitVersionChanges(self, REQUEST=None):
      #++ print "[%s._commitVersionChanges]: id=%s"%(self.meta_type,self.id)
      ##### Children ####
      delete = 0
      for child in self.getChildNodes():
        delete = delete + child._commitObjectVersionChanges(self,REQUEST)
      if delete: self.normalizeSortIdsAfterDelete()
      ##### Self ####
      parent = self.getParentNode()        
      delete = self._commitObjectVersionChanges(parent,REQUEST)
      url = self.absolute_url()
      if delete: 
        parent.normalizeSortIdsAfterDelete()
        url = parent.absolute_url()
      # Return new URL.
      return url

       
    ###############################################################################################
    #  VersionManagerContainer.manage_commitVersionChanges:	
    #
    #  Commit version changes.
    ###############################################################################################
    def manage_commitVersionChanges(self, lang, manage_lang, REQUEST):
      """ VersionManagerContainerContainer.manage_commitVersionChanges """
      
      ##### Notification ####

      #-- Recipient
      mto = ''
      name = self.getObjProperty('work_uid',REQUEST)
      print name
      ob = self
      while ob is not None:
        user_folder = ob.getUserFolder()
        if user_folder is not None:
          user = user_folder.getUser(name)
          if user is not None:
            mto = getattr(user,'email','')
        ob = ob.getParentNode()
      #-- Subject
      msubject \
        = self.getLangStr('ACTION_MANAGE_COMMITVERSIONCHANGES',manage_lang)%('%s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST))) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),REQUEST['AUTHENTICATED_USER'])
      #-- Body
      mbody \
        = self.getLangStr('ACTION_MANAGE_COMMITVERSIONCHANGES',manage_lang)%('%s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST))) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),REQUEST['AUTHENTICATED_USER']) \
        + '\n%s/manage'%(self.absolute_url()) \
        + '\n\n%s\n%s'%(self.getLangStr('WITH_BEST_REGARDS',manage_lang),str(REQUEST['AUTHENTICATED_USER']))

      # Send notification via MailHost.
      self.sendMail(REQUEST['AUTHENTICATED_USER'],mto,msubject,mbody)

      ##### Commit ####
      url = self._commitVersionChanges(REQUEST)

      ##### Return with message ####
      message = urllib.quote(self.getLangStr('ACTION_MANAGE_COMMITVERSIONCHANGES',manage_lang)%self.display_type(REQUEST))
      return REQUEST.RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(url,lang,manage_lang,message))


    """   
    ###############################################################################################
    #
    #   R O L L B A C K
    #
    ###############################################################################################
    """   
    
    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer._rollbackVersionChanges
    # ---------------------------------------------------------------------------------------------
    def _rollbackVersionChanges(self, REQUEST):
      #++ print "[%s._rollbackVersionChanges]: id=%s"%(self.meta_type,self.id)
      ##### Children ####
      delete = 0
      for child in self.getChildNodes():
        delete = delete + child._rollbackObjectVersionChanges(self,REQUEST)
      if delete: self.normalizeSortIdsAfterDelete()
      ##### Self ####
      parent = self.getParentNode()
      delete = self._rollbackObjectVersionChanges(parent,REQUEST)
      url = self.absolute_url()
      if delete: 
        parent.normalizeSortIdsAfterDelete()
        url = parent.absolute_url()
      # Return new URL.
      return url
       

    ###############################################################################################
    #  VersionManagerContainer.manage_rejectVersionChanges:	
    #
    #  Reject version changes.
    ###############################################################################################
    def manage_rejectVersionChanges(self, lang, manage_lang, REQUEST):
      """ VersionManagerContainer.manage_rejectVersionChanges """

      ##### Notification ####

      #-- Recipient
      mto = ''
      name = self.getObjProperty('work_uid',REQUEST)
      print name
      ob = self
      while ob is not None:
        user_folder = ob.getUserFolder()
        if user_folder:
          user = user_folder.getUser(name)
          if user is not None:
            mto = getattr(user,'email','')
        ob = ob.getParentNode()
      #-- Subject
      msubject \
        = self.getLangStr('ACTION_MANAGE_REJECTVERSIONCHANGES',manage_lang)%('%s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST))) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),REQUEST['AUTHENTICATED_USER'])
      #-- Body
      mbody \
        = self.getLangStr('ACTION_MANAGE_REJECTVERSIONCHANGES',manage_lang)%('%s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST))) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),REQUEST['AUTHENTICATED_USER']) \
        + '\n%s/manage'%(self.absolute_url()) \
        + '\n\n%s\n%s'%(self.getLangStr('WITH_BEST_REGARDS',manage_lang),str(REQUEST['AUTHENTICATED_USER']))

      # Send notification via MailHost.
      self.sendMail(REQUEST['AUTHENTICATED_USER'],mto,msubject,mbody)

      ##### Set object state ####
      self.resetObjWorkflowEvents(REQUEST)
      setObjState(self,'STATE_REJECTED',lang)
      setObjWorkflowEventChange(self,REQUEST)
      
      ##### Return with message ####
      message = urllib.quote(self.getLangStr('ACTION_MANAGE_REJECTVERSIONCHANGES',manage_lang)%self.display_type(REQUEST))
      return REQUEST.RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))

    
    ###############################################################################################
    #  VersionManagerContainer.manage_rollbackVersionChanges:	
    #
    #  Rollback version changes.
    ###############################################################################################
    def manage_rollbackVersionChanges(self, lang, manage_lang, REQUEST):
      """ VersionManagerContainer.manage_rollbackVersionChanges """

      ##### Notification ####

      #-- Recipient
      mto = ''
      name = self.getObjProperty('work_uid',REQUEST)
      print name
      ob = self
      while ob is not None:
        user_folder = ob.getUserFolder()
        if user_folder:
          user = user_folder.getUser(name)
          if user is not None:
            mto = getattr(user,'email','')
        ob = ob.getParentNode()
      #-- Subject
      msubject \
        = self.getLangStr('ACTION_MANAGE_ROLLBACKVERSIONCHANGES',manage_lang)%('%s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST))) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),REQUEST['AUTHENTICATED_USER'])
      #-- Body
      mbody \
        = self.getLangStr('ACTION_MANAGE_ROLLBACKVERSIONCHANGES',manage_lang)%('%s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST))) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),REQUEST['AUTHENTICATED_USER']) \
        + '\n%s/manage'%(self.absolute_url()) \
        + '\n\n%s\n%s'%(self.getLangStr('WITH_BEST_REGARDS',manage_lang),str(REQUEST['AUTHENTICATED_USER']))

      # Send notification via MailHost.
      self.sendMail(REQUEST['AUTHENTICATED_USER'],mto,msubject,mbody)

      ##### Rollback ####
      url = self._rollbackVersionChanges(REQUEST)

      ##### Set object state ####
      self.resetObjWorkflowEvents(REQUEST)
      setObjState(self,'STATE_ROLLEDBACK',lang)
      setObjWorkflowEventChange(self,REQUEST)
      
      ##### Return with message ####
      message = urllib.quote(self.getLangStr('ACTION_MANAGE_ROLLBACKVERSIONCHANGES',manage_lang)%self.display_type(REQUEST))
      return REQUEST.RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(url,lang,manage_lang,message))


    """   
    ###############################################################################################
    #
    #   R E Q U E S T
    #
    ###############################################################################################
    """   
        
    ###############################################################################################
    #  VersionManagerContainer.manage_requestVersionChanges:	
    #
    #  Request commit of version changes.
    ###############################################################################################
    def manage_requestVersionChanges(self, lang, manage_lang, REQUEST):
        """ VersionManagerContainer.manage_requestVersionChanges """

	##### Set object state ####
        self.resetObjWorkflowEvents(REQUEST)
	setObjState(self,'STATE_REQUESTED',lang)
	setObjWorkflowEventChange(self,REQUEST)
            
        ##### Notifications ####
            
        #-- Recipients
        recipients = []
        role = 'ZMSEditor'
        if REQUEST['AUTHENTICATED_USER'].has_role('ZMSEditor'):
          role = 'ZMSAdministrator'
        ob = self
        while ob is not None:
          user_folder = ob.getUserFolder()
          if user_folder:
            for user in user_folder.getUsers():
              if user.has_role(role) and \
                 len(getattr(user,'email','')) > 0 and \
                 lang in getattr(user,'langs',[]):
                recipients.append({'userid':user.getId(),'email':user.email})
          ob = ob.getParentNode()
            
        #-- Body
          
        # Init message body.
        mbody \
          = self.getLangStr('ACTION_MANAGE_REQUESTVERSIONCHANGES',manage_lang)%('%s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST))) \
          + ' %s %s '%(self.getLangStr('BY',manage_lang),REQUEST['AUTHENTICATED_USER']) \
          + '\n%s/manage'%(self.absolute_url())
        # Build message body.
        mbody = mbody + self._requestVersionChanges(self,lang,manage_lang,REQUEST)
        mbody = mbody + '\n'
        for child in self.getChildNodes():
          mbody = mbody + child._requestVersionChanges(self,lang,manage_lang,REQUEST)
        # With best regards.                
        mbody = mbody + '\n\n%s\n%s'%(self.getLangStr('WITH_BEST_REGARDS',manage_lang),str(REQUEST['AUTHENTICATED_USER']))

        #-- Subject
        msubject \
          = self.getLangStr('ACTION_MANAGE_REQUESTVERSIONCHANGES',manage_lang)%('%s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST))) \
          + ' %s %s '%(self.getLangStr('BY',manage_lang),REQUEST['AUTHENTICATED_USER'])

        for recipient in recipients:
          
          #-- Recipient
          mto = recipient['email']
            
          # Send notification via MailHost.
          self.sendMail(REQUEST['AUTHENTICATED_USER'],mto,msubject,mbody)
            
        # Return with message.
        message = urllib.quote(self.getLangStr('ACTION_MANAGE_REQUESTVERSIONCHANGES',manage_lang)%self.display_type(REQUEST))
        return REQUEST.RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))

###################################################################################################
