###################################################################################################
# zmsgraphic.py
#
# Implementation of class ZMSGraphic (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTML, HTMLFile
from OFS.Folder import Folder
import string 
import urllib
# Product Imports.
from zmsobject import ZMSObject
import _blobfields
import _fileutil
import _linklib
import _metadata
import _objtypes
import _textformatmanager
import zmslinkcontainer


###################################################################################################
###################################################################################################
###   
###   C o n s t r u c t o r ( s )
###   
###################################################################################################
###################################################################################################

def manage_addZMSGraphic(self, lang, manage_lang, _sort_id, REQUEST, RESPONSE):
  """ manage_addZMSGraphic """
  
  ##### Create ####
  obj = ZMSGraphic(self.getNewId(),_sort_id+1)
  self._setObject(obj.id, obj)
  
  for oItem in self.objectValues():
    if oItem.id == obj.id:
      ##### Object State ####
      oItem.setObjStateNew(REQUEST)
      ##### Init Properties ####
      for s_lang in self.getLanguages():
        oItem.setObjProperty('active',1,s_lang)
      ##### VersionManager ####
      oItem.__autoCommit__(REQUEST)
  
  ##### Normalize Sort-IDs ####
  self.normalizeSortIds()
  
  # Return with message.
  message = self.getLangStr('MSG_INSERTED',manage_lang)%obj.display_type(REQUEST)
  RESPONSE.redirect('%s/%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(self.absolute_url(),obj.id,lang,manage_lang,urllib.quote(message)))


###################################################################################################
###################################################################################################
###   
###   C l a s s
###   
###################################################################################################
###################################################################################################
class ZMSGraphic(ZMSObject, Folder, _metadata.Metadata, _textformatmanager.TextFormatObject):

    # Properties.
    # -----------
    meta_type = 'ZMSGraphic'
    icon = "misc_/zms/zmsgraphic.gif"
    icon_disabled = 'misc_/zms/zmsgraphic_disabled.gif'

    # Management Options.
    # -------------------
    manage_options = (  
	{'label': 'TAB_EDIT',    'action': 'manage_main'},
	{'label': 'TAB_PREVIEW', 'action': 'preview_html'}, # empty string defaults to index_html
	) 

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = (
		'manage','manage_main',
		'manage_moveObjUp','manage_moveObjDown',
		'manage_cutObjects','manage_copyObjects','manage_pasteObjs',
		'manage_properties','manage_changeProperties',
		'manage_rollbackVersionChanges','manage_requestVersionChanges',
		)
    __editorPermissions__ = (
		'manage_commitVersionChanges',
		)
    __administratorPermissions__ = (
		)
    __ac_permissions__=(
		('ZMS Author', __authorPermissions__),
		('ZMS Editor', __editorPermissions__),
		('ZMS Administrator', __administratorPermissions__),
		('Import/Export objects', ('manage_importexport','manage_import','manage_export',) ),
		)

    # Properties.
    # -----------
    __obj_attrs__ = {
        'sort_id':{'datatype':'string','native':1},
        'change_uid':{'datatype':'LangString','lang_inherit':-1},
        'change_dt':{'datatype':'LangDate','lang_inherit':-1},
        'active':{'datatype':'LangInt','type':'checkbox','lang_inherit':0},
        'format':{'datatype':'string'},
        'img':{'datatype':'LangMyImage'},
        'imghires':{'datatype':'LangMyImage'},
        'text':{'datatype':'LangString','type':'textarea'},
        'align':{'datatype':'string','default':'LEFT'},
        'textalign':{'datatype':'string','default':'LEFT'},
        'displaytype':{'datatype':'int','default':2},
        'img_attrs_spec':{'datatype':'string','default':'alt="" border="0"'},
        'attr_url':{'datatype':'LangString'},
        # Meta-Dictionaries        
        '$metadict':{'datatype':'MetaDict'},
    }


    # Management Interface.
    manage_main = HTMLFile('dtml/zmsgraphic/manage_main', globals())


    """
    ###############################################################################################    
    #
    #   CONSTRUCTOR
    #
    ###############################################################################################    
    """

    ###############################################################################################
    # ZMSGraphic.recurse_updateVersion:
    #
    # Update version.
    ###############################################################################################
    def recurse_updateVersion(self, lang, manage_lang, REQUEST):
        message = ''
        
        # Process super-objects.
        message = message + ZMSObject.recurse_updateVersion(self,lang,manage_lang,REQUEST)
        message = message + self.metadatarecurse_updateVersion(lang,manage_lang,REQUEST)
        message = message + zmslinkcontainer.recurse_updateVersion(self,lang,manage_lang,REQUEST)
        
        # Process object.
        # [...]
        
        # ---------------------------------------------------------------------
        # 26.11.2001 - 0.8 to 0.9: 
        # Meta-Dictionaries multilingual
        # ---------------------------------------------------------------------
        for s_attr in self.getMetadictAttrs(self.meta_type):
          if hasattr(self,s_attr):
            for s_lang in self.getLanguages():
              setattr(self,'%s_%s'%(s_attr,s_lang),getattr(self,s_attr))
            try: delattr(self,s_attr)
            except: pass
        
        # ---------------------------------------------------------------------
        # 11.01.2002 - 0.9 to 1.0: 
        # Blobs.
        # ---------------------------------------------------------------------
        _blobfields.synchronizeBlobFields(self)
        
        # ---------------------------------------------------------------------
        # 09.02.2002
        # Build #051: Make object-attributes multilingual.
        # ---------------------------------------------------------------------
        if getattr(self.getDocumentElement(),'build','') < '051':
          attrs = ['attr_url']
          for s_attr in attrs:
            if s_attr in self.__obj_attrs__.keys():
              for s_preview in [None,'preview']:
                req = {'preview':s_preview}
                ob = self.getObjVersion(req)
                if hasattr(ob,s_attr):
                  for s_lang in self.getLanguages():
                    setattr(ob,'%s_%s'%(s_attr,s_lang),getattr(ob,s_attr))
              self.__delObjProperty__(s_attr)
        
        # ---------------------------------------------------------------------
        # 15.04.2002
        # Build #070: Customizable Text-Formats.
        # ---------------------------------------------------------------------
        message = message + _textformatmanager.textformatrecurse_updateVersion(self,lang,manage_lang,REQUEST)
        
        # ---------------------------------------------------------------------
        # 27.05.2002
        # Build #076: Alter img_attrs_spec.
        # ---------------------------------------------------------------------
        if getattr(self.getDocumentElement(),'build','') < '076':
          for s_preview in [None,'preview']:
            req = {'preview':s_preview}
            ob = self.getObjVersion(req)
            img_attrs_spec = getattr(ob,'img_attrs_spec','')
            for s_attr in ['alt=""','border="0"']:
              if string.find(img_attrs_spec,s_attr) < 0:
                if len(img_attrs_spec) > 0:
                  img_attrs_spec = img_attrs_spec + ' '
                img_attrs_spec = img_attrs_spec + s_attr
            setattr(ob,'img_attrs_spec',img_attrs_spec)
        
        # Return with message.
        return message


    """
    ###############################################################################################    
    #
    #   Eigenschaften
    #
    ###############################################################################################    
    """

    ###############################################################################################
    #  ZMSGraphic.manage_changeProperties: 
    #
    #  Change Graphic properties.
    ###############################################################################################
    def manage_changeProperties(self, lang, manage_lang, REQUEST, RESPONSE): 
      """ ZMSGraphic.manage_changeProperties """
      
      message = ''
      if REQUEST.get('btn','') != self.getLangStr('BTN_CANCEL',manage_lang):
        
        ##### Object State #####
        self.setObjStateModified(REQUEST)
        
        ##### Properties #####
        self.setReqProperty('text',REQUEST)
        self.setReqProperty('active',REQUEST)
        self.setReqProperty('align',REQUEST) # Alignment (Image).
        self.setReqProperty('textalign',REQUEST) # Alignment (Text).
        self.setReqProperty('format',REQUEST) # Text-Format (e.g. Heading, Body)
        self.setReqProperty('img_attrs_spec',REQUEST) # Special Attributes (e.g. usemap).
        self.setReqProperty('displaytype',REQUEST) # Displaytype (positioning of Image and Text).
        self.setReqProperty('img',REQUEST)
        self.setReqProperty('imghires',REQUEST)
        self.setReqProperty('attr_url',REQUEST)
        
        ##### Thumbnail #####
        img = self.getObjProperty('img',REQUEST)
        imghires = self.getObjProperty('imghires',REQUEST)
        if img is None and \
           imghires is not None and \
           self.getConfProperty('InstalledProducts.pil',0) and \
           self.getConfProperty('InstalledProducts.pil.thumbnail',0):
          print "[%s.manage_changeProperties]: Create Image.Thumbnail from Image.HighRes..."%(self.meta_type)
          img = _fileutil.createThumbnail(imghires,self.getConfProperty('InstalledProducts.pil.thumbnail.max',100))
          self.setObjProperty('img',img,lang)
        
        ##### Blobs #####
        _blobfields.synchronizeBlobFields(self)
        
        ##### Metadata #####
        self.setMetadata(lang,manage_lang,REQUEST)
        
        ##### VersionManager ####
        self.__autoCommit__(REQUEST)
        
        ##### Message #####
        message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      # Return with message.
      self.checkIn(REQUEST)
      return RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s#_%s'%(self.getParentNode().absolute_url(),lang,manage_lang,urllib.quote(message),self.id))


    # ---------------------------------------------------------------------------------------------
    #	ZMSGraphic.isPageElement
    # -------------------------------------------------------------------------------------------------
    def isPageElement(self): return 1

    # ---------------------------------------------------------------------------------------------
    #	ZMSGraphic.isActive
    # -------------------------------------------------------------------------------------------------
    def isActive(self,REQUEST): return self.getObjProperty('active',REQUEST)

    # ---------------------------------------------------------------------------------------------
    #	ZMSGraphic.getTitlealt
    # -------------------------------------------------------------------------------------------------
    def getTitlealt(self,REQUEST): return self.display_type(REQUEST)

    # ---------------------------------------------------------------------------------------------
    #	ZMSGraphic.getText
    # -------------------------------------------------------------------------------------------------
    def getText(self,REQUEST): return self.getObjProperty('text',REQUEST,'')

    # ---------------------------------------------------------------------------------------------
    #	ZMSGraphic.getFormat
    # -------------------------------------------------------------------------------------------------
    def getFormat(self,REQUEST): return self.getObjProperty('format',REQUEST)

    # ---------------------------------------------------------------------------------------------
    #	ZMSGraphic.getImage
    # -------------------------------------------------------------------------------------------------
    def getImage(self,REQUEST): return self.getObjProperty('img',REQUEST)

    # ---------------------------------------------------------------------------------------------
    #	ZMSGraphic.getUrl
    # -------------------------------------------------------------------------------------------------
    def getUrl(self, REQUEST): return _linklib.getUrl(self,'attr_url',REQUEST)


    """
    ###############################################################################################
    ###  
    ###  HTML-Darstellung 
    ### 
    ###############################################################################################
    """

    ###############################################################################################
    #  ZMSGraphic._getBodyContent:
    #
    #  HTML presentation of Graphic (and Caption).
    ###############################################################################################
    def _getBodyContent(self, REQUEST, thumbnail=0):
      # Retrieve properties.
      displaytype = self.getObjProperty('displaytype',REQUEST)
      imgattr      = 'img'
      imghiresattr = 'imghires'
      imgalign     = self.getObjProperty('align',REQUEST)
      imgurl       = self.getUrl(REQUEST)
      imgthumb     = thumbnail
      imgspecial   = self.getObjProperty('img_attrs_spec',REQUEST)
      imgclass	   = None
      text         = self.getText(REQUEST)
      textalign    = self.getObjProperty('textalign',REQUEST)
      textformat   = self.getFormat(REQUEST)
      textclass    = None
      # Return <html>-presentation.
      return self.renderDisplaytype(displaytype,imgattr,imghiresattr,imgalign,imgurl,imgthumb,imgspecial,imgclass,text,textalign,textformat,textclass,REQUEST)

    ###############################################################################################
    #  ZMSGraphic.renderShort:
    #
    #  Renders short presentation of Graphic.
    ###############################################################################################
    def renderShort(self,REQUEST):
      return self._getBodyContent(REQUEST,thumbnail=1)

    ###############################################################################################    
    # ZMSGraphic.printHtml:
    #
    # Renders print presentation of Graphic.
    ###############################################################################################    
    def printHtml(self, level, sectionize, sectionizer, REQUEST):
      html = ''
      html = html + '<table cellspacing="0" cellpadding="2" border="0" width="100%">'
      
      # Image
      img = self.getObjProperty('img',REQUEST)
      imgattr = 'img'
      # Image (High-Resolution)
      imghires = self.getObjProperty('imghires',REQUEST)
      if imghires is not None: 
        img = imghires
        imgattr = 'imghires'
      
      # <IMG>
      if img is not None:
        img_src = img.getHref(REQUEST)
        img_width = str(img.width)
        img_height = str(img.height)
        html = html + '<tr>'
        html = html + '<td align="center"><img src="%s" alt="" border="0" width="%s" height="%s"></td>'%(img_src,img_width,img_height)
        html = html + '</tr>'
      
      # <CAPTION>
      text = self.getText(REQUEST)
      textformat = self.getFormat(REQUEST)
      if len(text) > 0:
        pre = "<u>" + self.display_type(REQUEST) + " " + str(sectionizer) + ":</u> "
        text = self.renderPrint(pre,textformat,text,REQUEST)
        html = html + '<tr>'
        html = html + '<td align="center">'
        html = html + '<table cellspacing="0" cellpadding="2" border="0">'
        html = html + '<tr>'
        html = html + '<td>%s</td>'%(text)
        html = html + '</tr>'
        html = html + '</table>'
        html = html + '</td>'
        html = html + '</tr>'
      
      html = html + '</table>'
      return html

    ###############################################################################################    
    #	ZMSGraphic.catalogText:
    #
    #	Catalog text.
    ###############################################################################################    
    def catalogText(self, REQUEST):
      s = ""
      s = s + self.getText(REQUEST) + " "
      return s

###############################################################################################
